"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""


import textwrap

import salt.modules.rh_service as rh_service
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase

RET = [
    "hostname",
    "mountall",
    "network-interface",
    "network-manager",
    "salt-api",
    "salt-master",
    "salt-minion",
]


class RhServiceTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.rh_service
    """

    def setup_loader_modules(self):
        return {
            rh_service: {
                "_upstart_disable": None,
                "_upstart_enable": None,
                "_upstart_is_enabled": None,
            }
        }

    @staticmethod
    def _m_lst():
        """
        Return value for [].
        """
        return MagicMock(return_value=[])

    @staticmethod
    def _m_ret():
        """
        Return value for RET.
        """
        return MagicMock(return_value=RET)

    @staticmethod
    def _m_bool(bol=True):
        """
        Return Bool value.
        """
        return MagicMock(return_value=bol)

    def test__chkconfig_is_enabled(self):
        """
        test _chkconfig_is_enabled function
        """
        name = "atd"
        chkconfig_out = textwrap.dedent(
            """\

            {}           0:off   1:off   2:off   3:on    4:on    5:on    6:off
            """.format(
                name
            )
        )
        xinetd_out = textwrap.dedent(
            """\
            xinetd based services:
                    {}  on
            """.format(
                name
            )
        )

        with patch.object(rh_service, "_runlevel", MagicMock(return_value=3)):
            mock_run = MagicMock(return_value={"retcode": 0, "stdout": chkconfig_out})
            with patch.dict(rh_service.__salt__, {"cmd.run_all": mock_run}):
                self.assertTrue(rh_service._chkconfig_is_enabled(name))
                self.assertFalse(rh_service._chkconfig_is_enabled(name, 2))
                self.assertTrue(rh_service._chkconfig_is_enabled(name, 3))

            mock_run = MagicMock(return_value={"retcode": 0, "stdout": xinetd_out})
            with patch.dict(rh_service.__salt__, {"cmd.run_all": mock_run}):
                self.assertTrue(rh_service._chkconfig_is_enabled(name))
                self.assertTrue(rh_service._chkconfig_is_enabled(name, 2))
                self.assertTrue(rh_service._chkconfig_is_enabled(name, 3))

    # 'get_enabled' function tests: 1

    def test_get_enabled(self):
        """
        Test if it return the enabled services. Use the ``limit``
        param to restrict results to services of that type.
        """
        with patch.object(rh_service, "_upstart_services", self._m_ret()):
            with patch.object(
                rh_service, "_upstart_is_enabled", MagicMock(return_value=False)
            ):
                self.assertListEqual(rh_service.get_enabled("upstart"), [])

        mock_run = MagicMock(return_value="salt stack")
        with patch.dict(rh_service.__salt__, {"cmd.run": mock_run}):
            with patch.object(rh_service, "_sysv_services", self._m_ret()):
                with patch.object(rh_service, "_sysv_is_enabled", self._m_bool()):
                    self.assertListEqual(rh_service.get_enabled("sysvinit"), RET)

                    with patch.object(rh_service, "_upstart_services", self._m_lst()):
                        with patch.object(
                            rh_service,
                            "_upstart_is_enabled",
                            MagicMock(return_value=True),
                        ):
                            self.assertListEqual(rh_service.get_enabled(), RET)

    # 'get_disabled' function tests: 1

    def test_get_disabled(self):
        """
        Test if it return the disabled services. Use the ``limit``
        param to restrict results to services of that type.
        """
        with patch.object(rh_service, "_upstart_services", self._m_ret()):
            with patch.object(
                rh_service, "_upstart_is_enabled", MagicMock(return_value=False)
            ):
                self.assertListEqual(rh_service.get_disabled("upstart"), RET)

        mock_run = MagicMock(return_value="salt stack")
        with patch.dict(rh_service.__salt__, {"cmd.run": mock_run}):
            with patch.object(rh_service, "_sysv_services", self._m_ret()):
                with patch.object(rh_service, "_sysv_is_enabled", self._m_bool(False)):
                    self.assertListEqual(rh_service.get_disabled("sysvinit"), RET)

                    with patch.object(rh_service, "_upstart_services", self._m_lst()):
                        with patch.object(
                            rh_service,
                            "_upstart_is_enabled",
                            MagicMock(return_value=False),
                        ):
                            self.assertListEqual(rh_service.get_disabled(), RET)

    # 'get_all' function tests: 1

    def test_get_all(self):
        """
        Test if it return all installed services. Use the ``limit``
        param to restrict results to services of that type.
        """
        with patch.object(rh_service, "_upstart_services", self._m_ret()):
            self.assertListEqual(rh_service.get_all("upstart"), RET)

        with patch.object(rh_service, "_sysv_services", self._m_ret()):
            self.assertListEqual(rh_service.get_all("sysvinit"), RET)

            with patch.object(rh_service, "_upstart_services", self._m_lst()):
                self.assertListEqual(rh_service.get_all(), RET)

    # 'available' function tests: 1

    def test_available(self):
        """
        Test if it return True if the named service is available.
        """
        with patch.object(rh_service, "_service_is_upstart", self._m_bool()):
            self.assertTrue(rh_service.available("salt-api", "upstart"))

        with patch.object(rh_service, "_service_is_sysv", self._m_bool()):
            self.assertTrue(rh_service.available("salt-api", "sysvinit"))

            with patch.object(rh_service, "_service_is_upstart", self._m_bool()):
                self.assertTrue(rh_service.available("salt-api"))

    # 'missing' function tests: 1

    def test_missing(self):
        """
        Test if it return True if the named service is not available.
        """
        with patch.object(rh_service, "_service_is_upstart", self._m_bool(False)):
            self.assertTrue(rh_service.missing("sshd", "upstart"))

            with patch.object(rh_service, "_service_is_sysv", self._m_bool(False)):
                self.assertTrue(rh_service.missing("sshd"))

        with patch.object(rh_service, "_service_is_sysv", self._m_bool()):
            self.assertFalse(rh_service.missing("sshd", "sysvinit"))

            with patch.object(rh_service, "_service_is_upstart", self._m_bool()):
                self.assertFalse(rh_service.missing("sshd"))

    # 'start' function tests: 1

    def test_start(self):
        """
        Test if it start the specified service.
        """
        with patch.object(rh_service, "_service_is_upstart", self._m_bool()):
            with patch.dict(rh_service.__salt__, {"cmd.retcode": self._m_bool(False)}):
                self.assertTrue(rh_service.start("salt-api"))

    # 'stop' function tests: 1

    def test_stop(self):
        """
        Test if it stop the specified service.
        """
        with patch.object(rh_service, "_service_is_upstart", self._m_bool()):
            with patch.dict(rh_service.__salt__, {"cmd.retcode": self._m_bool(False)}):
                self.assertTrue(rh_service.stop("salt-api"))

    # 'restart' function tests: 1

    def test_restart(self):
        """
        Test if it restart the specified service.
        """
        with patch.object(rh_service, "_service_is_upstart", self._m_bool()):
            with patch.dict(rh_service.__salt__, {"cmd.retcode": self._m_bool(False)}):
                self.assertTrue(rh_service.restart("salt-api"))

    # 'reload_' function tests: 1

    def test_reload(self):
        """
        Test if it reload the specified service.
        """
        with patch.object(rh_service, "_service_is_upstart", self._m_bool()):
            with patch.dict(rh_service.__salt__, {"cmd.retcode": self._m_bool(False)}):
                self.assertTrue(rh_service.reload_("salt-api"))

    # 'status' function tests: 1

    def test_status(self):
        """
        Test if it return the status for a service,
        returns a bool whether the service is running.
        """
        with patch.object(rh_service, "_service_is_upstart", self._m_bool()):
            mock_run = MagicMock(return_value="start/running")
            with patch.dict(rh_service.__salt__, {"cmd.run": mock_run}):
                self.assertTrue(rh_service.status("salt-api"))

        with patch.object(rh_service, "_service_is_upstart", self._m_bool(False)):
            with patch.dict(rh_service.__salt__, {"status.pid": self._m_bool()}):
                self.assertTrue(rh_service.status("salt-api", sig=True))

            mock_ret = MagicMock(return_value=0)
            with patch.dict(rh_service.__salt__, {"cmd.retcode": mock_ret}):
                self.assertTrue(rh_service.status("salt-api"))

    # 'enable' function tests: 1

    def test_enable(self):
        """
        Test if it enable the named service to start at boot.
        """
        mock_bool = MagicMock(side_effect=[True, True, False])
        with patch.object(rh_service, "_service_is_upstart", mock_bool):
            with patch.object(
                rh_service, "_upstart_is_enabled", MagicMock(return_value=True)
            ):
                with patch.object(
                    rh_service, "_upstart_enable", MagicMock(return_value=False)
                ):
                    self.assertFalse(rh_service.enable("salt-api"))
                with patch.object(
                    rh_service, "_upstart_enable", MagicMock(return_value=True)
                ):
                    self.assertTrue(rh_service.enable("salt-api"))

            with patch.object(rh_service, "_sysv_enable", self._m_bool()):
                self.assertTrue(rh_service.enable("salt-api"))

    # 'disable' function tests: 1

    def test_disable(self):
        """
        Test if it disable the named service to start at boot.
        """
        mock_bool = MagicMock(side_effect=[True, True, False])
        with patch.object(rh_service, "_service_is_upstart", mock_bool):
            with patch.object(
                rh_service, "_upstart_is_enabled", MagicMock(return_value=True)
            ):
                with patch.object(
                    rh_service, "_upstart_disable", MagicMock(return_value=False)
                ):
                    self.assertFalse(rh_service.disable("salt-api"))
                with patch.object(
                    rh_service, "_upstart_disable", MagicMock(return_value=True)
                ):
                    self.assertTrue(rh_service.disable("salt-api"))

            with patch.object(rh_service, "_sysv_disable", self._m_bool()):
                self.assertTrue(rh_service.disable("salt-api"))

    # 'enabled' function tests: 1

    def test_enabled(self):
        """
        Test if it check to see if the named service is enabled
        to start on boot.
        """
        mock_bool = MagicMock(side_effect=[True, False])
        with patch.object(rh_service, "_service_is_upstart", mock_bool):
            with patch.object(
                rh_service, "_upstart_is_enabled", MagicMock(return_value=False)
            ):
                self.assertFalse(rh_service.enabled("salt-api"))

            with patch.object(rh_service, "_sysv_is_enabled", self._m_bool()):
                self.assertTrue(rh_service.enabled("salt-api"))

    # 'disabled' function tests: 1

    def test_disabled(self):
        """
        Test if it check to see if the named service is disabled
        to start on boot.
        """
        mock_bool = MagicMock(side_effect=[True, False])
        with patch.object(rh_service, "_service_is_upstart", mock_bool):
            with patch.object(
                rh_service, "_upstart_is_enabled", MagicMock(return_value=False)
            ):
                self.assertTrue(rh_service.disabled("salt-api"))

            with patch.object(rh_service, "_sysv_is_enabled", self._m_bool(False)):
                self.assertTrue(rh_service.disabled("salt-api"))



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import salt.states.keystone as keystone
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class KeystoneTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.keystone
    """

    def setup_loader_modules(self):
        return {keystone: {}}

    # 'user_present' function tests: 1

    def test_user_present(self):
        """
        Test to ensure that the keystone user is present
        with the specified properties.
        """
        name = "nova"
        password = "$up3rn0v4"
        email = "nova@domain.com"
        tenant = "demo"

        ret = {"name": name, "result": False, "changes": {}, "comment": ""}

        mock_f = MagicMock(return_value=False)
        mock_lst = MagicMock(return_value=["Error"])
        with patch.dict(keystone.__salt__, {"keystone.tenant_get": mock_lst}):
            comt = 'Tenant / project "{}" does not exist'.format(tenant)
            ret.update({"comment": comt})
            self.assertDictEqual(
                keystone.user_present(name, password, email, tenant), ret
            )

        mock_dict = MagicMock(
            side_effect=[
                {name: {"email": "a@a.com"}},
                {name: {"email": email, "enabled": False}},
                {name: {"email": email, "enabled": True}},
                {name: {"email": email, "enabled": True}},
                {"Error": "error"},
                {"Error": "error"},
            ]
        )
        mock_l = MagicMock(return_value={tenant: {"id": "abc"}})
        with patch.dict(
            keystone.__salt__,
            {
                "keystone.user_get": mock_dict,
                "keystone.tenant_get": mock_l,
                "keystone.user_verify_password": mock_f,
                "keystone.user_create": mock_f,
            },
        ):
            with patch.dict(keystone.__opts__, {"test": True}):
                comt = 'User "{}" will be updated'.format(name)
                ret.update(
                    {
                        "comment": comt,
                        "result": None,
                        "changes": {
                            "Email": "Will be updated",
                            "Enabled": "Will be True",
                            "Password": "Will be updated",
                        },
                    }
                )
                self.assertDictEqual(keystone.user_present(name, password, email), ret)

                ret.update(
                    {
                        "comment": comt,
                        "result": None,
                        "changes": {
                            "Enabled": "Will be True",
                            "Password": "Will be updated",
                        },
                    }
                )
                self.assertDictEqual(keystone.user_present(name, password, email), ret)

                ret.update(
                    {
                        "comment": comt,
                        "result": None,
                        "changes": {
                            "Tenant": 'Will be added to "demo" tenant',
                            "Password": "Will be updated",
                        },
                    }
                )
                self.assertDictEqual(
                    keystone.user_present(name, password, email, tenant), ret
                )

                ret.update(
                    {
                        "comment": comt,
                        "result": None,
                        "changes": {"Password": "Will be updated"},
                    }
                )
                self.assertDictEqual(keystone.user_present(name, password, email), ret)

                comt = 'Keystone user "nova" will be added'
                ret.update(
                    {
                        "comment": comt,
                        "result": None,
                        "changes": {"User": "Will be created"},
                    }
                )
                self.assertDictEqual(keystone.user_present(name, password, email), ret)

            with patch.dict(keystone.__opts__, {"test": False}):
                comt = "Keystone user {} has been added".format(name)
                ret.update(
                    {"comment": comt, "result": True, "changes": {"User": "Created"}}
                )
                self.assertDictEqual(keystone.user_present(name, password, email), ret)

    # 'user_absent' function tests: 1

    def test_user_absent(self):
        """
        Test to ensure that the keystone user is absent.
        """
        name = "nova"

        ret = {
            "name": name,
            "changes": {},
            "result": True,
            "comment": 'User "{}" is already absent'.format(name),
        }

        mock_lst = MagicMock(side_effect=[["Error"], []])
        with patch.dict(keystone.__salt__, {"keystone.user_get": mock_lst}):
            self.assertDictEqual(keystone.user_absent(name), ret)

            with patch.dict(keystone.__opts__, {"test": True}):
                comt = 'User "{}" will be deleted'.format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(keystone.user_absent(name), ret)

    # 'tenant_present' function tests: 1

    def test_tenant_present(self):
        """
        Test to ensures that the keystone tenant exists
        """
        name = "nova"
        description = "OpenStack Compute Service"

        ret = {
            "name": name,
            "changes": {},
            "result": True,
            "comment": 'Tenant / project "{}" already exists'.format(name),
        }

        mock_dict = MagicMock(
            side_effect=[
                {name: {"description": "desc"}},
                {name: {"description": description, "enabled": False}},
                {"Error": "error"},
                {"Error": "error"},
            ]
        )
        mock_t = MagicMock(return_value=True)
        with patch.dict(
            keystone.__salt__,
            {"keystone.tenant_get": mock_dict, "keystone.tenant_create": mock_t},
        ):
            with patch.dict(keystone.__opts__, {"test": True}):
                comt = 'Tenant / project "{}" will be updated'.format(name)
                ret.update(
                    {
                        "comment": comt,
                        "result": None,
                        "changes": {"Description": "Will be updated"},
                    }
                )
                self.assertDictEqual(keystone.tenant_present(name), ret)

                comt = 'Tenant / project "{}" will be updated'.format(name)
                ret.update(
                    {
                        "comment": comt,
                        "result": None,
                        "changes": {"Enabled": "Will be True"},
                    }
                )
                self.assertDictEqual(keystone.tenant_present(name, description), ret)

                comt = 'Tenant / project "{}" will be added'.format(name)
                ret.update(
                    {
                        "comment": comt,
                        "result": None,
                        "changes": {"Tenant": "Will be created"},
                    }
                )
                self.assertDictEqual(keystone.tenant_present(name), ret)

            with patch.dict(keystone.__opts__, {"test": False}):
                comt = 'Tenant / project "{}" has been added'.format(name)
                ret.update(
                    {"comment": comt, "result": True, "changes": {"Tenant": "Created"}}
                )
                self.assertDictEqual(keystone.tenant_present(name), ret)

    # 'tenant_absent' function tests: 1

    def test_tenant_absent(self):
        """
        Test to ensure that the keystone tenant is absent.
        """
        name = "nova"

        ret = {
            "name": name,
            "changes": {},
            "result": True,
            "comment": 'Tenant / project "{}" is already absent'.format(name),
        }

        mock_lst = MagicMock(side_effect=[["Error"], []])
        with patch.dict(keystone.__salt__, {"keystone.tenant_get": mock_lst}):
            self.assertDictEqual(keystone.tenant_absent(name), ret)

            with patch.dict(keystone.__opts__, {"test": True}):
                comt = 'Tenant / project "{}" will be deleted'.format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(keystone.tenant_absent(name), ret)

    # 'role_present' function tests: 1

    def test_role_present(self):
        """
        Test to ensures that the keystone role exists
        """
        name = "nova"

        ret = {
            "name": name,
            "changes": {},
            "result": True,
            "comment": 'Role "{}" already exists'.format(name),
        }

        mock_lst = MagicMock(side_effect=[[], ["Error"]])
        with patch.dict(keystone.__salt__, {"keystone.role_get": mock_lst}):
            self.assertDictEqual(keystone.role_present(name), ret)

            with patch.dict(keystone.__opts__, {"test": True}):
                comt = 'Role "{}" will be added'.format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(keystone.role_present(name), ret)

    # 'role_absent' function tests: 1

    def test_role_absent(self):
        """
        Test to ensure that the keystone role is absent.
        """
        name = "nova"

        ret = {
            "name": name,
            "changes": {},
            "result": True,
            "comment": 'Role "{}" is already absent'.format(name),
        }

        mock_lst = MagicMock(side_effect=[["Error"], []])
        with patch.dict(keystone.__salt__, {"keystone.role_get": mock_lst}):
            self.assertDictEqual(keystone.role_absent(name), ret)

            with patch.dict(keystone.__opts__, {"test": True}):
                comt = 'Role "{}" will be deleted'.format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(keystone.role_absent(name), ret)

    # 'service_present' function tests: 1

    def test_service_present(self):
        """
        Test to ensure service present in Keystone catalog
        """
        name = "nova"
        service_type = "compute"

        ret = {
            "name": name,
            "changes": {},
            "result": True,
            "comment": 'Service "{}" already exists'.format(name),
        }

        mock_lst = MagicMock(side_effect=[[], ["Error"]])
        with patch.dict(keystone.__salt__, {"keystone.service_get": mock_lst}):
            self.assertDictEqual(keystone.service_present(name, service_type), ret)

            with patch.dict(keystone.__opts__, {"test": True}):
                comt = 'Service "{}" will be added'.format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(keystone.service_present(name, service_type), ret)

    # 'service_absent' function tests: 1

    def test_service_absent(self):
        """
        Test to ensure that the service doesn't exist in Keystone catalog
        """
        name = "nova"

        ret = {
            "name": name,
            "changes": {},
            "result": True,
            "comment": 'Service "{}" is already absent'.format(name),
        }

        mock_lst = MagicMock(side_effect=[["Error"], []])
        with patch.dict(keystone.__salt__, {"keystone.service_get": mock_lst}):
            self.assertDictEqual(keystone.service_absent(name), ret)

            with patch.dict(keystone.__opts__, {"test": True}):
                comt = 'Service "{}" will be deleted'.format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(keystone.service_absent(name), ret)

    # 'endpoint_present' function tests: 1

    def test_endpoint_present(self):
        """
        Test to ensure the specified endpoints exists for service
        """
        name = "nova"
        region = "RegionOne"

        ret = {"name": name, "changes": {}, "result": True, "comment": ""}

        endpoint = {
            "adminurl": None,
            "region": None,
            "internalurl": None,
            "publicurl": None,
            "id": 1,
            "service_id": None,
        }

        mock_lst = MagicMock(
            side_effect=[endpoint, ["Error"], {"id": 1, "service_id": None}, []]
        )
        mock = MagicMock(return_value=True)
        with patch.dict(
            keystone.__salt__,
            {"keystone.endpoint_get": mock_lst, "keystone.endpoint_create": mock},
        ):

            comt = 'Endpoint for service "{}" already exists'.format(name)
            ret.update({"comment": comt, "result": True, "changes": {}})
            self.assertDictEqual(keystone.endpoint_present(name), ret)

            with patch.dict(keystone.__opts__, {"test": True}):
                comt = 'Endpoint for service "{}" will be added'.format(name)
                ret.update(
                    {
                        "comment": comt,
                        "result": None,
                        "changes": {"Endpoint": "Will be created"},
                    }
                )
                self.assertDictEqual(keystone.endpoint_present(name), ret)

                comt = 'Endpoint for service "{}" already exists'.format(name)
                ret.update({"comment": comt, "result": True, "changes": {}})
                self.assertDictEqual(keystone.endpoint_present(name), ret)

            with patch.dict(keystone.__opts__, {"test": False}):
                comt = 'Endpoint for service "{}" has been added'.format(name)
                ret.update({"comment": comt, "result": True, "changes": True})
                self.assertDictEqual(keystone.endpoint_present(name), ret)

    # 'endpoint_absent' function tests: 1

    def test_endpoint_absent(self):
        """
        Test to ensure that the endpoint for a service doesn't
         exist in Keystone catalog
        """
        name = "nova"
        region = "RegionOne"
        comment = 'Endpoint for service "{}" is already absent'.format(name)
        ret = {"name": name, "changes": {}, "result": True, "comment": comment}

        mock_lst = MagicMock(side_effect=[[], ["Error"]])
        with patch.dict(keystone.__salt__, {"keystone.endpoint_get": mock_lst}):
            self.assertDictEqual(keystone.endpoint_absent(name, region), ret)

            with patch.dict(keystone.__opts__, {"test": True}):
                comt = 'Endpoint for service "{}" will be deleted'.format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(keystone.endpoint_absent(name, region), ret)



import salt.modules.win_powercfg as powercfg
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, call, patch
from tests.support.unit import TestCase


class PowerCfgTestCase(TestCase, LoaderModuleMockMixin):
    """
    Validate the powercfg state
    """

    def setup_loader_modules(self):
        return {powercfg: {"__grains__": {"osrelease": 8}}}

    query_output = """Subgroup GUID: 238c9fa8-0aad-41ed-83f4-97be242c8f20  (Hibernate)
                GUID Alias: SUB_SLEEP
                Power Setting GUID: 29f6c1db-86da-48c5-9fdb-f2b67b1f44da  (Hibernate after)
                GUID Alias: HIBERNATEIDLE
                Minimum Possible Setting: 0x00000000
                Maximum Possible Setting: 0xffffffff
                Possible Settings increment: 0x00000001
                Possible Settings units: Seconds
                Current AC Power Setting Index: 0x00000708
                Current DC Power Setting Index: 0x00000384"""

    def test_set_monitor_timeout(self):
        """
        Test to make sure we can set the monitor timeout value
        """
        mock = MagicMock(return_value=0)
        mock.side_effect = [
            "Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)",
            self.query_output,
        ]

        mock_retcode = MagicMock(return_value=0)

        with patch.dict(powercfg.__salt__, {"cmd.run": mock}):
            with patch.dict(powercfg.__salt__, {"cmd.retcode": mock_retcode}):
                powercfg.set_monitor_timeout(0, "dc")
                mock.assert_called_once_with(
                    "powercfg /getactivescheme", python_shell=False
                )
                mock_retcode.assert_called_once_with(
                    "powercfg /setdcvalueindex 381b4222-f694-41f0-9685-ff5bb260df2e"
                    " SUB_VIDEO VIDEOIDLE 0",
                    python_shell=False,
                )

    def test_set_disk_timeout(self):
        """
        Test to make sure we can set the disk timeout value
        """
        mock = MagicMock()
        mock.side_effect = [
            "Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)",
            self.query_output,
        ]

        mock_retcode = MagicMock(return_value=0)

        with patch.dict(powercfg.__salt__, {"cmd.run": mock}):
            with patch.dict(powercfg.__salt__, {"cmd.retcode": mock_retcode}):
                powercfg.set_disk_timeout(0, "dc")
                mock.assert_called_once_with(
                    "powercfg /getactivescheme", python_shell=False
                )
                mock_retcode.assert_called_once_with(
                    "powercfg /setdcvalueindex 381b4222-f694-41f0-9685-ff5bb260df2e"
                    " SUB_DISK DISKIDLE 0",
                    python_shell=False,
                )

    def test_set_standby_timeout(self):
        """
        Test to make sure we can set the standby timeout value
        """
        mock = MagicMock(return_value=0)
        mock.side_effect = [
            "Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)",
            self.query_output,
        ]

        mock_retcode = MagicMock(return_value=0)

        with patch.dict(powercfg.__salt__, {"cmd.run": mock}):
            with patch.dict(powercfg.__salt__, {"cmd.retcode": mock_retcode}):
                powercfg.set_standby_timeout(0, "dc")
                mock.assert_called_once_with(
                    "powercfg /getactivescheme", python_shell=False
                )
                mock_retcode.assert_called_once_with(
                    "powercfg /setdcvalueindex 381b4222-f694-41f0-9685-ff5bb260df2e"
                    " SUB_SLEEP STANDBYIDLE 0",
                    python_shell=False,
                )

    def test_set_hibernate_timeout(self):
        """
        Test to make sure we can set the hibernate timeout value
        """
        mock = MagicMock(return_value=0)
        mock.side_effect = [
            "Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)",
            self.query_output,
        ]

        mock_retcode = MagicMock(return_value=0)

        with patch.dict(powercfg.__salt__, {"cmd.run": mock}):
            with patch.dict(powercfg.__salt__, {"cmd.retcode": mock_retcode}):
                powercfg.set_hibernate_timeout(0, "dc")
                mock.assert_called_once_with(
                    "powercfg /getactivescheme", python_shell=False
                )
                mock_retcode.assert_called_once_with(
                    "powercfg /setdcvalueindex 381b4222-f694-41f0-9685-ff5bb260df2e"
                    " SUB_SLEEP HIBERNATEIDLE 0",
                    python_shell=False,
                )

    def test_get_monitor_timeout(self):
        """
        Test to make sure we can get the monitor timeout value
        """
        mock = MagicMock()
        mock.side_effect = [
            "Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)",
            self.query_output,
        ]

        with patch.dict(powercfg.__salt__, {"cmd.run": mock}):
            ret = powercfg.get_monitor_timeout()
            calls = [
                call("powercfg /getactivescheme", python_shell=False),
                call(
                    "powercfg /q 381b4222-f694-41f0-9685-ff5bb260df2e SUB_VIDEO"
                    " VIDEOIDLE",
                    python_shell=False,
                ),
            ]
            mock.assert_has_calls(calls)

            self.assertEqual({"ac": 30, "dc": 15}, ret)

    def test_get_disk_timeout(self):
        """
        Test to make sure we can get the disk timeout value
        """
        mock = MagicMock()
        mock.side_effect = [
            "Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)",
            self.query_output,
        ]

        with patch.dict(powercfg.__salt__, {"cmd.run": mock}):
            ret = powercfg.get_disk_timeout()
            calls = [
                call("powercfg /getactivescheme", python_shell=False),
                call(
                    "powercfg /q 381b4222-f694-41f0-9685-ff5bb260df2e SUB_DISK"
                    " DISKIDLE",
                    python_shell=False,
                ),
            ]
            mock.assert_has_calls(calls)

            self.assertEqual({"ac": 30, "dc": 15}, ret)

    def test_get_standby_timeout(self):
        """
        Test to make sure we can get the standby timeout value
        """
        mock = MagicMock()
        mock.side_effect = [
            "Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)",
            self.query_output,
        ]

        with patch.dict(powercfg.__salt__, {"cmd.run": mock}):
            ret = powercfg.get_standby_timeout()
            calls = [
                call("powercfg /getactivescheme", python_shell=False),
                call(
                    "powercfg /q 381b4222-f694-41f0-9685-ff5bb260df2e SUB_SLEEP"
                    " STANDBYIDLE",
                    python_shell=False,
                ),
            ]
            mock.assert_has_calls(calls)

            self.assertEqual({"ac": 30, "dc": 15}, ret)

    def test_get_hibernate_timeout(self):
        """
        Test to make sure we can get the hibernate timeout value
        """
        mock = MagicMock()
        mock.side_effect = [
            "Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)",
            self.query_output,
        ]

        with patch.dict(powercfg.__salt__, {"cmd.run": mock}):
            ret = powercfg.get_hibernate_timeout()
            calls = [
                call("powercfg /getactivescheme", python_shell=False),
                call(
                    "powercfg /q 381b4222-f694-41f0-9685-ff5bb260df2e SUB_SLEEP"
                    " HIBERNATEIDLE",
                    python_shell=False,
                ),
            ]
            mock.assert_has_calls(calls)

            self.assertEqual({"ac": 30, "dc": 15}, ret)

    def test_windows_7(self):
        """
        Test to make sure we can get the hibernate timeout value on windows 7
        """
        mock = MagicMock()
        mock.side_effect = [
            "Power Scheme GUID: 381b4222-f694-41f0-9685-ff5bb260df2e  (Balanced)",
            self.query_output,
        ]

        with patch.dict(powercfg.__salt__, {"cmd.run": mock}):
            with patch.dict(powercfg.__grains__, {"osrelease": "7"}):
                ret = powercfg.get_hibernate_timeout()
                calls = [
                    call("powercfg /getactivescheme", python_shell=False),
                    call(
                        "powercfg /q 381b4222-f694-41f0-9685-ff5bb260df2e SUB_SLEEP",
                        python_shell=False,
                    ),
                ]
                mock.assert_has_calls(calls)

                self.assertEqual({"ac": 30, "dc": 15}, ret)

    def test_set_hibernate_timeout_scheme(self):
        """
        Test to make sure we can set the hibernate timeout value
        """
        mock = MagicMock(return_value=0)
        mock.side_effect = [self.query_output]

        with patch.dict(powercfg.__salt__, {"cmd.retcode": mock}):
            powercfg.set_hibernate_timeout(0, "dc", scheme="SCHEME_MIN")
            mock.assert_called_once_with(
                "powercfg /setdcvalueindex SCHEME_MIN SUB_SLEEP HIBERNATEIDLE 0",
                python_shell=False,
            )

    def test_get_hibernate_timeout_scheme(self):
        """
        Test to make sure we can get the hibernate timeout value with a
        specified scheme
        """
        mock = MagicMock()
        mock.side_effect = [self.query_output]

        with patch.dict(powercfg.__salt__, {"cmd.run": mock}):
            ret = powercfg.get_hibernate_timeout(scheme="SCHEME_MIN")
            mock.assert_called_once_with(
                "powercfg /q SCHEME_MIN SUB_SLEEP HIBERNATEIDLE", python_shell=False
            )

            self.assertEqual({"ac": 30, "dc": 15}, ret)



"""
    :codeauthor: Thomas Stoner <tmstoner@cisco.com>
"""

# Copyright (c) 2018 Cisco and/or its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import salt.modules.nxos_upgrade as nxos_upgrade
from salt.exceptions import CommandExecutionError, NxosError
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase
from tests.unit.modules.nxos.nxos_n3k import N3KPlatform
from tests.unit.modules.nxos.nxos_n5k import N5KPlatform
from tests.unit.modules.nxos.nxos_n7k import N7KPlatform
from tests.unit.modules.nxos.nxos_n36k import N36KPlatform
from tests.unit.modules.nxos.nxos_n93k import N93KPlatform
from tests.unit.modules.nxos.nxos_n93klxc import N93KLXCPlatform
from tests.unit.modules.nxos.nxos_n95k import N95KPlatform

# pylint: disable-msg=C0103
# pylint: disable-msg=C0301
# pylint: disable-msg=E1101
# pylint: disable-msg=R0904


class NxosUpgradeTestCase(TestCase, LoaderModuleMockMixin):

    """Test cases for salt.modules.nxos_upgrade"""

    platform_list = None

    @staticmethod
    def assert_platform_upgrade(condition, platform):

        """Assert platform upgrade condition and display appropriate chassis & images upon assertion failure"""

        assert bool(condition), "{}: Upgrade {} -> {}".format(
            platform.chassis, platform.cimage, platform.nimage
        )

    def setup_loader_modules(self):

        """Define list of platforms for Unit Test"""

        self.platform_list = [
            N3KPlatform(cimage="nxos.7.0.3.F3.3.bin", nimage="nxos.9.2.1.255.bin"),
            N36KPlatform(cimage="nxos.9.1.2.50.bin", nimage="nxos.9.2.2.50.bin"),
            N5KPlatform(
                ckimage="n6000-uk9-kickstart.7.3.0.N1.1.bin",
                cimage="n6000-uk9.7.3.0.N1.1.bin",
                nkimage="n6000-uk9-kickstart.7.3.3.N2.1.bin",
                nimage="n6000-uk9.7.3.3.N2.1.bin",
            ),
            N7KPlatform(
                ckimage="n7000-s2-kickstart.7.3.0.D1.1.bin",
                cimage="n7000-s2-dk9.7.3.0.D1.1.bin",
                nkimage="n7000-s2-kickstart.8.3.1.112.gbin",
                nimage="n7000-s2-dk9.8.3.1.112.gbin",
            ),
            N93KPlatform(cimage="nxos.7.0.3.I7.4.bin", nimage="nxos.7.0.3.I7.5.bin"),
            N93KPlatform(cimage="nxos.7.0.3.I7.5.bin", nimage="nxos.7.0.3.I7.5.bin"),
            N93KLXCPlatform(cimage="nxos.7.0.3.I7.4.bin", nimage="nxos.7.0.3.I7.5.bin"),
            N95KPlatform(cimage="nxos.7.0.3.I7.4.bin", nimage="nxos.9.2.2.14.bin"),
        ]

        return {nxos_upgrade: {}}

    def tearDown(self):
        del self.platform_list

    @staticmethod
    def test_check_upgrade_impact_input_validation():

        """UT: nxos_upgrade module:check_upgrade_impact method - input validation"""

        result = nxos_upgrade.check_upgrade_impact("dummy-platform-image.bin", issu=1)
        assert "Input Error" in result

    @staticmethod
    def test_upgrade_input_validation():

        """UT: nxos_upgrade module:upgrade method - input validation"""

        result = nxos_upgrade.upgrade("dummy-platform-image.bin", issu=1)
        assert "Input Error" in result

    def test_check_upgrade_impact_backend_processing_error_500(self):

        """UT: nxos_upgrade module:check_upgrade_impact method - error HTTP code 500"""

        for platform in self.platform_list:
            if platform.backend_processing_error_500:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=platform.backend_processing_error_500
                        )
                    },
                ):
                    result = nxos_upgrade.check_upgrade_impact(platform.nimage)
                    self.assert_platform_upgrade(
                        result["backend_processing_error"], platform
                    )
                    self.assert_platform_upgrade(not result["succeeded"], platform)

    def test_check_upgrade_impact_internal_server_error_400_invalid_command(self):

        """UT: nxos_upgrade module:check_upgrade_impact method - invalid command error HTTP code 400"""

        for platform in self.platform_list:
            if platform.bad_request_client_error_400_invalid_command_dict:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=platform.bad_request_client_error_400_invalid_command_dict
                        )
                    },
                ):
                    result = nxos_upgrade.check_upgrade_impact(platform.nimage)
                    self.assert_platform_upgrade(result["invalid_command"], platform)
                    self.assert_platform_upgrade(not result["succeeded"], platform)

    def test_check_upgrade_impact_internal_server_error_400_in_progress(self):

        """UT: nxos_upgrade module:check_upgrade_impact method - in-progress error HTTP code 400"""

        for platform in self.platform_list:
            if platform.bad_request_client_error_400_in_progress_dict:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=platform.bad_request_client_error_400_in_progress_dict
                        )
                    },
                ):
                    result = nxos_upgrade.check_upgrade_impact(platform.nimage)
                    self.assert_platform_upgrade(result["installing"], platform)
                    self.assert_platform_upgrade(not result["succeeded"], platform)

    def test_check_upgrade_impact_internal_server_error_500(self):

        """UT: nxos_upgrade module:check_upgrade_impact method - internal server error HTTP code 500"""

        for platform in self.platform_list:
            if platform.internal_server_error_500:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=platform.internal_server_error_500
                        )
                    },
                ):
                    result = nxos_upgrade.check_upgrade_impact(platform.nimage)
                    self.assert_platform_upgrade(
                        platform.internal_server_error_500 in result["error_data"],
                        platform,
                    )
                    self.assert_platform_upgrade(
                        result["backend_processing_error"], platform
                    )
                    self.assert_platform_upgrade(not result["succeeded"], platform)

    def test_check_upgrade_impact_non_disruptive_success(self):

        """UT: nxos_upgrade module:check_upgrade_impact method - non-disruptive success"""

        for platform in self.platform_list:
            if platform.install_all_non_disruptive_success:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=platform.install_all_non_disruptive_success
                        )
                    },
                ):
                    result = nxos_upgrade.check_upgrade_impact(platform.nimage)
                    self.assert_platform_upgrade(
                        result["upgrade_non_disruptive"], platform
                    )
                    self.assert_platform_upgrade(result["succeeded"], platform)
                    self.assert_platform_upgrade(result["module_data"], platform)

    def test_check_upgrade_impact_disruptive_success(self):

        """UT: nxos_upgrade module:check_upgrade_impact method - disruptive success"""

        for platform in self.platform_list:
            if platform.install_all_disruptive_success:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=platform.install_all_disruptive_success
                        )
                    },
                ):
                    result = nxos_upgrade.check_upgrade_impact(platform.nimage)
                    self.assert_platform_upgrade(
                        result["upgrade_required"] == platform.upgrade_required,
                        platform,
                    )
                    self.assert_platform_upgrade(
                        not result["upgrade_non_disruptive"], platform
                    )
                    self.assert_platform_upgrade(not result["succeeded"], platform)
                    self.assert_platform_upgrade(
                        result["upgrade_in_progress"], platform
                    )
                    self.assert_platform_upgrade(result["module_data"], platform)

    def test_upgrade_show_install_all_impact_no_module_data(self):

        """UT: nxos_upgrade module: upgrade method - no module data"""

        for platform in self.platform_list:
            if platform.show_install_all_impact_no_module_data:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=platform.show_install_all_impact_no_module_data
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(platform.nimage, issu=False)
                    self.assert_platform_upgrade(not result["succeeded"], platform)
                    self.assert_platform_upgrade(
                        result["error_data"] == result["upgrade_data"], platform
                    )

    def test_upgrade_invalid_command(self):

        """UT: nxos_upgrade module:upgrade method - invalid command"""

        for platform in self.platform_list:
            if platform.invalid_command:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {"nxos.sendline": MagicMock(return_value=platform.invalid_command)},
                ):
                    result = nxos_upgrade.upgrade(platform.nimage, platform.nkimage)
                    self.assert_platform_upgrade(result["error_data"], platform)
                    self.assert_platform_upgrade(not result["succeeded"], platform)

    def test_upgrade_install_in_progress(self):

        """UT: nxos_upgrade module:upgrade method - in-progress"""

        for platform in self.platform_list:
            if platform.show_install_all_impact_in_progress:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=platform.show_install_all_impact_in_progress
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(platform.nimage, platform.nkimage)
                    self.assert_platform_upgrade(result["error_data"], platform)
                    self.assert_platform_upgrade(not result["succeeded"], platform)

    def test_upgrade_install_in_progress_terminal_dont_ask(self):

        """UT: nxos_upgrade module:upgrade method - in-progress (terminal don't-ask)"""

        for platform in self.platform_list:
            if platform.invalid_command:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=[
                                {},
                                platform.show_install_all_impact_in_progress,
                            ]
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(platform.nimage, platform.nkimage)
                    self.assert_platform_upgrade(result["error_data"], platform)
                    self.assert_platform_upgrade(not result["succeeded"], platform)

    def test_upgrade_install_in_progress_sans_terminal_dont_ask(self):

        """UT: nxos_upgrade module:upgrade method - in-progress (sans terminal don't-ask)"""

        for platform in self.platform_list:
            if platform.invalid_command:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=[platform.show_install_all_impact_in_progress]
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(platform.nimage, platform.nkimage)
                    self.assert_platform_upgrade(result["error_data"], platform)
                    self.assert_platform_upgrade(not result["succeeded"], platform)

    def test_upgrade_internal_server_error_500(self):

        """UT: nxos_upgrade module:upgrade method - internal server error 500"""

        for platform in self.platform_list:
            if platform.backend_processing_error_500:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            return_value=platform.internal_server_error_500
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(platform.nimage)
                    self.assert_platform_upgrade(result["error_data"], platform)
                    self.assert_platform_upgrade(
                        result["backend_processing_error"], platform
                    )
                    self.assert_platform_upgrade(not result["succeeded"], platform)

    def test_upgrade_install_all_disruptive(self):

        """UT: nxos_upgrade module:upgrade method - install all disruptive"""

        for platform in self.platform_list:
            if platform.show_install_all_impact:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            side_effect=[
                                platform.show_install_all_impact,
                                platform.install_all_disruptive_success,
                            ]
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(
                        platform.nimage, platform.nkimage, issu=False
                    )
                    self.assert_platform_upgrade(not result["error_data"], platform)
                    if platform.upgrade_required:
                        self.assert_platform_upgrade(
                            result["upgrade_in_progress"], platform
                        )
                    else:
                        self.assert_platform_upgrade(
                            not result["upgrade_in_progress"], platform
                        )

    def test_upgrade_install_all_non_disruptive(self):

        """UT: nxos_upgrade module:upgrade method - install all non-disruptive"""

        for platform in self.platform_list:
            if platform.show_install_all_impact_non_disruptive:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            side_effect=[
                                platform.show_install_all_impact_non_disruptive,
                                platform.install_all_non_disruptive_success,
                            ]
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(
                        platform.nimage, platform.nkimage, issu=True
                    )
                    self.assert_platform_upgrade(not result["error_data"], platform)
                    self.assert_platform_upgrade(result["succeeded"], platform)

    def test_upgrade_CommandExecutionError_Exception(self):

        """UT: nxos_upgrade module:upgrade method - raise CommandExecutionError exception #1"""

        for platform in self.platform_list:
            if platform.invalid_command:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            side_effect=CommandExecutionError(
                                {
                                    "rejected_input": "invalid CLI command",
                                    "message": "CLI excution error",
                                    "code": "400",
                                    "cli_error": platform.invalid_command,
                                }
                            )
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(
                        platform.nimage, platform.nkimage, issu=False
                    )
                    self.assert_platform_upgrade(result["error_data"], platform)
                    self.assert_platform_upgrade(result["invalid_command"], platform)
                    self.assert_platform_upgrade(not result["succeeded"], platform)

    def test_upgrade_CommandExecutionError_Exception2(self):

        """UT: nxos_upgrade module:upgrade method - raise CommandExecutionError exception #2"""

        for platform in self.platform_list:
            if platform.invalid_command:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            side_effect=[
                                platform.show_install_all_impact,
                                CommandExecutionError(
                                    {
                                        "rejected_input": "invalid CLI command",
                                        "message": "CLI excution error",
                                        "code": "400",
                                        "cli_error": platform.invalid_command,
                                    }
                                ),
                            ]
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(
                        platform.nimage, platform.nkimage, issu=False
                    )
                    if platform.upgrade_required:
                        self.assert_platform_upgrade(result["error_data"], platform)
                        self.assert_platform_upgrade(
                            result["invalid_command"], platform
                        )
                        self.assert_platform_upgrade(not result["succeeded"], platform)
                    else:
                        self.assert_platform_upgrade(result["succeeded"], platform)

    def test_upgrade_NxosError_Exception(self):

        """UT: nxos_upgrade module:upgrade method - raise NxosError exception"""

        for platform in self.platform_list:
            if platform.internal_server_error_500:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            side_effect=[
                                platform.show_install_all_impact,
                                NxosError(platform.internal_server_error_500),
                            ]
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(
                        platform.nimage, platform.nkimage, issu=False
                    )
                    if platform.upgrade_required:
                        self.assert_platform_upgrade(
                            result["upgrade_in_progress"], platform
                        )
                        self.assert_platform_upgrade(not result["succeeded"], platform)
                    else:
                        self.assert_platform_upgrade(
                            not result["upgrade_in_progress"], platform
                        )
                        self.assert_platform_upgrade(result["succeeded"], platform)

    def test_upgrade_NxosError_Exception2(self):

        """UT: nxos_upgrade module:upgrade method - raise NxosError exception #2"""

        for platform in self.platform_list:
            if platform.internal_server_error_500:
                with patch.dict(
                    nxos_upgrade.__salt__,
                    {
                        "nxos.sendline": MagicMock(
                            side_effect=[
                                platform.show_install_all_impact,
                                NxosError(
                                    "{'Error Message': 'Not Found', 'Code': 404}"
                                ),
                            ]
                        )
                    },
                ):
                    result = nxos_upgrade.upgrade(
                        platform.nimage, platform.nkimage, issu=False
                    )
                    if platform.upgrade_required:
                        self.assert_platform_upgrade(
                            result["upgrade_in_progress"], platform
                        )
                        self.assert_platform_upgrade(not result["succeeded"], platform)
                    else:
                        self.assert_platform_upgrade(
                            not result["upgrade_in_progress"], platform
                        )
                        self.assert_platform_upgrade(result["succeeded"], platform)



import pytest
import salt.engines
from tests.support.mock import MagicMock, patch


def test_engine_module_name():
    engine = salt.engines.Engine("foobar", {}, "foobar.start", {}, {}, {}, {})
    assert engine.name == "foobar"


def test_engine_title_set():
    engine = salt.engines.Engine("foobar", {}, "foobar.start", {}, {}, {}, {})
    with patch("salt.utils.process.appendproctitle", MagicMock()) as mm:
        with pytest.raises(KeyError):
            # The method does not exist so a KeyError will be raised.
            engine.run()
        mm.assert_called_with("foobar")



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""


import salt.modules.netscaler as netscaler
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MockJson(Exception):
    """
    Mock Json class
    """

    @staticmethod
    def loads(content):
        """
        Mock loads method
        """
        return content

    @staticmethod
    def dumps(dumps):
        """
        Mock dumps method
        """
        return dumps


class MockNSNitroError(Exception):
    """
    Mock NSNitroError class
    """

    def __init__(self, message="error"):
        self._message = message
        super().__init__(self.message)

    def _get_message(self):
        """
        get_message method
        """
        return self._message

    def _set_message(self, message):
        """
        set_message method
        """
        self._message = message

    message = property(_get_message, _set_message)


class MockNSNitro:
    """
    Mock NSNitro class
    """

    flag = None

    def __init__(self, host, user, passwd, bol):
        pass

    @staticmethod
    def login():
        """
        Mock login method
        """
        return True

    @staticmethod
    def logout():
        """
        Mock logout method
        """
        return True


class MockNSServiceGroup:
    """
    Mock NSServiceGroup class
    """

    def __init__(self):
        self.sg_name = None

    def set_servicegroupname(self, sg_name):
        """
        Mock set_servicegroupname method
        """
        self.sg_name = sg_name
        return MockNSServiceGroup()

    @staticmethod
    def get(obj, servicegroup):
        """
        Mock get method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSServiceGroup()

    @staticmethod
    def add(obj, servicegroup):
        """
        Mock add method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSServiceGroup()

    @staticmethod
    def delete(obj, servicegroup):
        """
        Mock delete method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSServiceGroup()

    @staticmethod
    def get_servers(obj, servicegroup):
        """
        Mock get_servers method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return [MockNSServiceGroup()]

    @staticmethod
    def enable_server(obj, servicegroup):
        """
        Mock enable_server method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSServiceGroup()

    @staticmethod
    def disable_server(obj, servicegroup):
        """
        Mock disable_server method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSServiceGroup()

    @staticmethod
    def get_servername():
        """
        Mock get_servername method
        """
        return "serviceGroupName"

    @staticmethod
    def get_state():
        """
        Mock get_state method
        """
        return "ENABLED"

    @staticmethod
    def get_servicetype():
        """
        Mock get_servicetype method
        """
        return ""

    @staticmethod
    def set_servicetype(bol):
        """
        Mock set_servicetype method
        """
        return bol


class MockNSServiceGroupServerBinding:
    """
    Mock NSServiceGroupServerBinding class
    """

    def __init__(self):
        self.sg_name = None

    def set_servername(self, sg_name):
        """
        Mock set_servername method
        """
        self.sg_name = sg_name
        return MockNSServiceGroupServerBinding()

    def set_servicegroupname(self, sg_name):
        """
        Mock set_servicegroupname method
        """
        self.sg_name = sg_name
        return MockNSServiceGroupServerBinding()

    def set_port(self, sg_name):
        """
        Mock set_port method
        """
        self.sg_name = sg_name
        return MockNSServiceGroupServerBinding()

    @staticmethod
    def add(obj, servicegroup):
        """
        Mock add method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSServiceGroupServerBinding()

    @staticmethod
    def delete(obj, servicegroup):
        """
        Mock delete method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSServiceGroupServerBinding()


class MockNSService:
    """
    Mock NSService class
    """

    def __init__(self):
        self.sg_name = None

    def set_name(self, sg_name):
        """
        Mock set_name method
        """
        self.sg_name = sg_name
        return MockNSService()

    @staticmethod
    def get(obj, servicegroup):
        """
        Mock get method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSService()

    @staticmethod
    def enable(obj, servicegroup):
        """
        Mock enable method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSService()

    @staticmethod
    def disable(obj, servicegroup):
        """
        Mock disable method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSService()

    @staticmethod
    def get_svrstate():
        """
        Mock get_svrstate method
        """
        return "UP"


class MockNSServer:
    """
    Mock NSServer class
    """

    flag = None

    def __init__(self):
        self.sg_name = None

    def set_name(self, sg_name):
        """
        Mock set_name method
        """
        self.sg_name = sg_name
        return MockNSServer()

    @staticmethod
    def get(obj, servicegroup):
        """
        Mock get method
        """
        return MockNSServer()

    @staticmethod
    def add(obj, servicegroup):
        """
        Mock add method
        """
        return MockNSServer()

    @staticmethod
    def delete(obj, servicegroup):
        """
        Mock delete method
        """
        return MockNSServer()

    @staticmethod
    def update(obj, servicegroup):
        """
        Mock update method
        """
        return MockNSServer()

    @staticmethod
    def enable(obj, servicegroup):
        """
        Mock enable method
        """
        return MockNSServer()

    @staticmethod
    def disable(obj, servicegroup):
        """
        Mock disable method
        """
        return MockNSServer()

    @staticmethod
    def get_ipaddress():
        """
        Mock get_ipaddress method
        """
        return ""

    @staticmethod
    def set_ipaddress(s_ip):
        """
        Mock set_ipaddress method
        """
        return s_ip

    def get_state(self):
        """
        Mock get_state method
        """
        if self.flag == 1:
            return ""
        elif self.flag == 2:
            return "DISABLED"
        return "ENABLED"


class MockNSLBVServer:
    """
    Mock NSLBVServer class
    """

    def __init__(self):
        self.sg_name = None

    def set_name(self, sg_name):
        """
        Mock set_name method
        """
        self.sg_name = sg_name
        return MockNSLBVServer()

    @staticmethod
    def get(obj, servicegroup):
        """
        Mock get method
        """
        return MockNSLBVServer()

    @staticmethod
    def set_ipv46(v_ip):
        """
        Mock set_ipv46 method
        """
        return v_ip

    @staticmethod
    def set_port(v_port):
        """
        Mock set_port method
        """
        return v_port

    @staticmethod
    def set_servicetype(v_type):
        """
        Mock set_servicetype method
        """
        return v_type

    @staticmethod
    def get_ipv46():
        """
        Mock get_ipv46 method
        """
        return ""

    @staticmethod
    def get_port():
        """
        Mock get_port method
        """
        return ""

    @staticmethod
    def get_servicetype():
        """
        Mock get_servicetype method
        """
        return ""

    @staticmethod
    def add(obj, servicegroup):
        """
        Mock add method
        """
        return MockNSLBVServer()

    @staticmethod
    def delete(obj, servicegroup):
        """
        Mock delete method
        """
        return MockNSLBVServer()


class MockNSLBVServerServiceGroupBinding:
    """
    Mock NSLBVServerServiceGroupBinding class
    """

    flag = None

    def __init__(self):
        self.sg_name = None

    def set_name(self, sg_name):
        """
        Mock set_name method
        """
        self.sg_name = sg_name
        return MockNSLBVServerServiceGroupBinding()

    @staticmethod
    def get(obj, servicegroup):
        """
        Mock get method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return [MockNSLBVServerServiceGroupBinding()]

    @staticmethod
    def get_servicegroupname():
        """
        Mock get_servicegroupname method
        """
        return "serviceGroupName"

    def set_servicegroupname(self, sg_name):
        """
        Mock set_servicegroupname method
        """
        self.sg_name = sg_name
        if self.flag:
            return None
        return MockNSLBVServerServiceGroupBinding()

    @staticmethod
    def add(obj, servicegroup):
        """
        Mock add method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSLBVServerServiceGroupBinding()

    @staticmethod
    def delete(obj, servicegroup):
        """
        Mock delete method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSLBVServerServiceGroupBinding()


class MockNSSSLVServerSSLCertKeyBinding:
    """
    Mock NSSSLVServerSSLCertKeyBinding class
    """

    def __init__(self):
        self.sg_name = None

    def set_vservername(self, sg_name):
        """
        Mock set_vservername method
        """
        self.sg_name = sg_name
        return MockNSSSLVServerSSLCertKeyBinding()

    @staticmethod
    def get(obj, servicegroup):
        """
        Mock get method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return [MockNSSSLVServerSSLCertKeyBinding()]

    @staticmethod
    def get_certkeyname():
        """
        Mock get_certkeyname method
        """
        return "serviceGroupName"

    def set_certkeyname(self, sg_name):
        """
        Mock set_certkeyname method
        """
        self.sg_name = sg_name
        return MockNSSSLVServerSSLCertKeyBinding()

    @staticmethod
    def add(obj, servicegroup):
        """
        Mock add method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSSSLVServerSSLCertKeyBinding()

    @staticmethod
    def delete(obj, servicegroup):
        """
        Mock delete method
        """
        if MockNSNitro.flag:
            raise MockNSNitroError
        return MockNSSSLVServerSSLCertKeyBinding()


class NetscalerTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.modules.netscaler
    """

    def setup_loader_modules(self):
        return {
            netscaler: {
                "NSNitro": MockNSNitro,
                "NSServiceGroup": MockNSServiceGroup,
                "NSServiceGroupServerBinding": MockNSServiceGroupServerBinding,
                "NSLBVServerServiceGroupBinding": MockNSLBVServerServiceGroupBinding,
                "NSService": MockNSService,
                "NSServer": MockNSServer,
                "NSLBVServer": MockNSLBVServer,
                "NSNitroError": MockNSNitroError,
                "NSSSLVServerSSLCertKeyBinding": MockNSSSLVServerSSLCertKeyBinding,
            }
        }

    # 'servicegroup_exists' function tests: 1

    def test_servicegroup_exists(self):
        """
        Tests if it checks if a service group exists
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            MockNSNitro.flag = None
            self.assertTrue(netscaler.servicegroup_exists("serviceGrpName"))

            self.assertFalse(
                netscaler.servicegroup_exists("serviceGrpName", sg_type="HTTP")
            )

            MockNSNitro.flag = True
            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(netscaler.servicegroup_exists("serGrpNme"))

    # 'servicegroup_add' function tests: 1

    def test_servicegroup_add(self):
        """
        Tests if it add a new service group
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertFalse(netscaler.servicegroup_add("serviceGroupName"))

            MockNSNitro.flag = True
            self.assertFalse(netscaler.servicegroup_add("serviceGroupName"))

            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(netscaler.servicegroup_add("serveGrpName"))

    # 'servicegroup_delete' function tests: 1

    def test_servicegroup_delete(self):
        """
        Tests if it delete a new service group
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            MockNSNitro.flag = None
            self.assertTrue(netscaler.servicegroup_delete("serviceGrpName"))

            mock = MagicMock(side_effect=[None, MockNSServiceGroup()])
            with patch.object(netscaler, "_servicegroup_get", mock):
                MockNSNitro.flag = True
                self.assertFalse(netscaler.servicegroup_delete("srGrpName"))

                with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                    self.assertFalse(netscaler.servicegroup_delete("sGNam"))

    # 'servicegroup_server_exists' function tests: 1

    def test_servicegroup_server_exists(self):
        """
        Tests if it check if a server:port combination
        is a member of a servicegroup
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertFalse(
                netscaler.servicegroup_server_exists(
                    "serviceGrpName", "serverName", "serverPort"
                )
            )

    # 'servicegroup_server_up' function tests: 1

    def test_servicegroup_server_up(self):
        """
        Tests if it check if a server:port combination
        is a member of a servicegroup
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertFalse(
                netscaler.servicegroup_server_up(
                    "serviceGrpName", "serverName", "serverPort"
                )
            )

    # 'servicegroup_server_enable' function tests: 1

    def test_servicegroup_server_enable(self):
        """
        Tests if it enable a server:port member of a servicegroup
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertFalse(
                netscaler.servicegroup_server_enable(
                    "serviceGrpName", "serverName", "serverPort"
                )
            )

            with patch.object(
                netscaler,
                "_servicegroup_get_server",
                MagicMock(return_value=MockNSServiceGroup()),
            ):
                MockNSNitro.flag = None
                self.assertTrue(
                    netscaler.servicegroup_server_enable(
                        "servGrpName", "serverName", "serPort"
                    )
                )

                with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                    self.assertFalse(
                        netscaler.servicegroup_server_enable(
                            "serGrpName", "serverName", "sPort"
                        )
                    )

    # 'servicegroup_server_disable' function tests: 1

    def test_sergrp_server_disable(self):
        """
        Tests if it disable a server:port member of a servicegroup
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertFalse(
                netscaler.servicegroup_server_disable(
                    "serviceGrpName", "serverName", "serverPort"
                )
            )

            with patch.object(
                netscaler,
                "_servicegroup_get_server",
                MagicMock(return_value=MockNSServiceGroup()),
            ):
                MockNSNitro.flag = None
                self.assertTrue(
                    netscaler.servicegroup_server_disable(
                        "serveGrpName", "serverName", "serPort"
                    )
                )

                with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                    self.assertFalse(
                        netscaler.servicegroup_server_disable(
                            "servGrpName", "serverName", "sPort"
                        )
                    )

    # 'servicegroup_server_add' function tests: 1

    def test_servicegroup_server_add(self):
        """
        Tests if it add a server:port member to a servicegroup
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(
                    netscaler.servicegroup_server_add(
                        "serGrpName", "serverName", "sPort"
                    )
                )

            MockNSNitro.flag = None
            self.assertTrue(
                netscaler.servicegroup_server_add(
                    "serGrpName", "serverName", "serverPort"
                )
            )

            mock = MagicMock(return_value=MockNSServiceGroupServerBinding())
            with patch.object(netscaler, "_servicegroup_get_server", mock):
                MockNSNitro.flag = True
                self.assertFalse(
                    netscaler.servicegroup_server_add(
                        "serviceGroupName", "serverName", "serPort"
                    )
                )

    # 'servicegroup_server_delete' function tests: 1

    def test_servicegroup_server_delete(self):
        """
        Tests if it remove a server:port member to a servicegroup
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(
                    netscaler.servicegroup_server_delete(
                        "servGrpName", "serverName", "sPort"
                    )
                )

            self.assertFalse(
                netscaler.servicegroup_server_delete(
                    "serviceGroupName", "serverName", "serverPort"
                )
            )

            mock = MagicMock(return_value=MockNSServiceGroupServerBinding())
            with patch.object(netscaler, "_servicegroup_get_server", mock):
                MockNSNitro.flag = None
                self.assertTrue(
                    netscaler.servicegroup_server_delete(
                        "serviceGroupName", "serverName", "serPort"
                    )
                )

    # 'service_up' function tests: 1

    def test_service_up(self):
        """
        Tests if it checks if a service is UP
        """
        mock = MagicMock(return_value=MockNSService())
        with patch.object(netscaler, "_service_get", mock):
            self.assertTrue(netscaler.service_up("serviceGrpName"))

    # 'service_exists' function tests: 1

    def test_service_exists(self):
        """
        Tests if it checks if a service is UP
        """
        mock = MagicMock(return_value=MockNSService())
        with patch.object(netscaler, "_service_get", mock):
            self.assertTrue(netscaler.service_exists("serviceGrpName"))

    # 'service_enable' function tests: 1

    def test_service_enable(self):
        """
        Tests if it enable a service
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertTrue(netscaler.service_enable("serviceGrpName"))

            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(netscaler.service_enable("serviceGrpName"))

                mock = MagicMock(return_value=MockNSService())
                with patch.object(netscaler, "_service_get", mock):
                    self.assertFalse(netscaler.service_enable("serGrpName"))

    # 'service_disable' function tests: 1

    def test_service_disable(self):
        """
        Tests if it disable a service
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertTrue(netscaler.service_disable("serviceGrpName"))

            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(netscaler.service_disable("serceGrpName"))

                mock = MagicMock(return_value=MockNSService())
                with patch.object(netscaler, "_service_get", mock):
                    self.assertFalse(netscaler.service_disable("seGrpName"))

    # 'server_exists' function tests: 1

    def test_server_exists(self):
        """
        Tests if it checks if a server exists
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertTrue(netscaler.server_exists("serviceGrpName"))

            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(netscaler.server_exists("serviceGrpName"))

            self.assertFalse(netscaler.server_exists("serviceGrpName", ip="1.0.0.1"))

            self.assertFalse(
                netscaler.server_exists("serviceGrpName", s_state="serverName")
            )

    # 'server_add' function tests: 1

    def test_server_add(self):
        """
        Tests if it add a server
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertFalse(netscaler.server_add("servGrpName", "1.0.0.1"))

            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(netscaler.server_add("serviceGrpName", "1.0.0.1"))

            mock = MagicMock(return_value=False)
            with patch.object(netscaler, "server_exists", mock):
                self.assertTrue(netscaler.server_add("serviceGrpName", "1.0.0.1"))

    # 'server_delete' function tests: 1

    def test_server_delete(self):
        """
        Tests if it delete a server
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertTrue(netscaler.server_delete("serviceGrpName"))

            mock = MagicMock(side_effect=[MockNSServer(), None])
            with patch.object(netscaler, "_server_get", mock):
                with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                    self.assertFalse(netscaler.server_delete("serGrpName"))

                self.assertFalse(netscaler.server_delete("serviceGrpName"))

    # 'server_update' function tests: 1

    def test_server_update(self):
        """
        Tests if it update a server's attributes
        """
        mock = MagicMock(
            side_effect=[None, MockNSServer(), MockNSServer(), MockNSServer()]
        )
        with patch.object(netscaler, "_server_get", mock):
            self.assertFalse(netscaler.server_update("seGrName", "1.0.0.1"))

            self.assertFalse(netscaler.server_update("serGrpName", ""))

            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(netscaler.server_update("serGrpName", "1.0.0.1"))

            mock = MagicMock(return_value="")
            with patch.dict(netscaler.__salt__, {"config.option": mock}):
                self.assertTrue(netscaler.server_update("serGrpName", "1.0.0.1"))

    # 'server_enabled' function tests: 1

    def test_server_enabled(self):
        """
        Tests if it check if a server is enabled globally
        """
        mock = MagicMock(return_value=MockNSServer())
        with patch.object(netscaler, "_server_get", mock):
            MockNSServer.flag = None
            self.assertTrue(netscaler.server_enabled("serGrpName"))

    # 'server_enable' function tests: 1

    def test_server_enable(self):
        """
        Tests if it enables a server globally
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertTrue(netscaler.server_enable("serGrpName"))

            MockNSServer.flag = 1
            self.assertTrue(netscaler.server_enable("serGrpName"))

            mock = MagicMock(side_effect=[MockNSServer(), None])
            with patch.object(netscaler, "_server_get", mock):
                with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                    self.assertFalse(netscaler.server_enable("serGrpName"))

                self.assertFalse(netscaler.server_enable("serGrpName"))

    # 'server_disable' function tests: 1

    def test_server_disable(self):
        """
        Tests if it disable a server globally
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertTrue(netscaler.server_disable("serGrpName"))

            MockNSServer.flag = 2
            self.assertTrue(netscaler.server_disable("serGrpName"))

            MockNSServer.flag = None
            mock = MagicMock(side_effect=[None, MockNSServer()])
            with patch.object(netscaler, "_server_get", mock):
                self.assertFalse(netscaler.server_disable("serGrpName"))

                with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                    self.assertFalse(netscaler.server_disable("serGrpName"))

    # 'vserver_exists' function tests: 1

    def test_vserver_exists(self):
        """
        Tests if it checks if a vserver exists
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertTrue(netscaler.vserver_exists("vserverName"))

            self.assertFalse(netscaler.vserver_exists("vserverName", v_ip="1.0.0.1"))

            self.assertFalse(
                netscaler.vserver_exists("vserrName", v_ip="", v_port="vserverPort")
            )

            self.assertFalse(
                netscaler.vserver_exists(
                    "vserrName", v_ip="", v_port="", v_type="vserverType"
                )
            )

            mock = MagicMock(return_value=None)
            with patch.object(netscaler, "_vserver_get", mock):
                self.assertFalse(netscaler.vserver_exists("vserverName"))

    # 'vserver_add' function tests: 1

    def test_vserver_add(self):
        """
        Tests if it add a new lb vserver
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertFalse(
                netscaler.vserver_add("alex.patate.chaude.443", "1.2.3.4", "443", "SSL")
            )

            mock = MagicMock(return_value=False)
            with patch.object(netscaler, "vserver_exists", mock):
                self.assertTrue(
                    netscaler.vserver_add(
                        "alex.pae.chaude.443", "1.2.3.4", "443", "SSL"
                    )
                )

                with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                    self.assertFalse(
                        netscaler.vserver_add("alex.chde.443", "1.2.3.4", "443", "SSL")
                    )

    # 'vserver_delete' function tests: 1

    def test_vserver_delete(self):
        """
        Tests if it delete a new lb vserver
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertTrue(netscaler.vserver_delete("alex.pe.chaude.443"))

            mock = MagicMock(side_effect=[None, MockNSLBVServer()])
            with patch.object(netscaler, "_vserver_get", mock):
                self.assertFalse(netscaler.vserver_delete("alex.chade.443"))

                with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                    self.assertFalse(netscaler.vserver_delete("al.cha.443"))

    # 'vserver_servicegroup_exists' function tests: 1

    def test_vser_sergrp_exists(self):
        """
        Tests if it checks if a servicegroup is tied to a vserver
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertTrue(
                netscaler.vserver_servicegroup_exists("vserverName", "serviceGroupName")
            )

    # 'vserver_servicegroup_add' function tests: 1

    def test_vserver_servicegroup_add(self):
        """
        Tests if it bind a servicegroup to a vserver
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            MockNSNitro.flag = None
            self.assertTrue(
                netscaler.vserver_servicegroup_add("vserverName", "serGroupName")
            )

            mock = MagicMock(side_effect=[MockNSLBVServerServiceGroupBinding(), None])
            with patch.object(netscaler, "vserver_servicegroup_exists", mock):
                self.assertFalse(
                    netscaler.vserver_servicegroup_add("vserName", "serGroupName")
                )

                with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                    self.assertFalse(
                        netscaler.vserver_servicegroup_add("vName", "serGroupName")
                    )

    # 'vserver_servicegroup_delete' function tests: 1

    def test_vser_sergrp_delete(self):
        """
        Tests if it unbind a servicegroup from a vserver
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertFalse(
                netscaler.vserver_servicegroup_delete("vservName", "serGroupName")
            )

            mock = MagicMock(return_value=MockNSLBVServerServiceGroupBinding())
            with patch.object(netscaler, "vserver_servicegroup_exists", mock):
                MockNSNitro.flag = None
                self.assertTrue(
                    netscaler.vserver_servicegroup_delete("vName", "serGroupName")
                )

                with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                    self.assertFalse(
                        netscaler.vserver_servicegroup_delete(
                            "vserverName", "serGroupName"
                        )
                    )

    # 'vserver_sslcert_exists' function tests: 1

    def test_vserver_sslcert_exists(self):
        """
        Tests if it checks if a SSL certificate is tied to a vserver
        """
        mock = MagicMock(return_value="")
        with patch.dict(netscaler.__salt__, {"config.option": mock}):
            self.assertTrue(
                netscaler.vserver_sslcert_exists("vserverName", "serviceGroupName")
            )

    # 'vserver_sslcert_add' function tests: 1

    def test_vserver_sslcert_add(self):
        """
        Tests if it binds a SSL certificate to a vserver
        """
        mock = MagicMock(side_effect=[MockNSSSLVServerSSLCertKeyBinding(), None, None])
        with patch.object(netscaler, "vserver_sslcert_exists", mock):
            self.assertFalse(netscaler.vserver_sslcert_add("vserName", "serGroupName"))

            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(netscaler.vserver_sslcert_add("vName", "serGrName"))

            mock = MagicMock(return_value="")
            with patch.dict(netscaler.__salt__, {"config.option": mock}):
                MockNSNitro.flag = None
                self.assertTrue(
                    netscaler.vserver_sslcert_add("vserverName", "serGroupName")
                )

    # 'vserver_sslcert_delete' function tests: 1

    def test_vserver_sslcert_delete(self):
        """
        Tests if it unbinds a SSL certificate from a vserver
        """
        mock = MagicMock(
            side_effect=[
                None,
                MockNSSSLVServerSSLCertKeyBinding(),
                MockNSSSLVServerSSLCertKeyBinding(),
            ]
        )
        with patch.object(netscaler, "vserver_sslcert_exists", mock):
            self.assertFalse(netscaler.vserver_sslcert_delete("vName", "serGrpName"))

            mock = MagicMock(return_value="")
            with patch.dict(netscaler.__salt__, {"config.option": mock}):
                MockNSNitro.flag = None
                self.assertTrue(
                    netscaler.vserver_sslcert_delete("vservName", "serGroupName")
                )

            with patch.object(netscaler, "_connect", MagicMock(return_value=None)):
                self.assertFalse(
                    netscaler.vserver_sslcert_delete("vserverName", "serGroupName")
                )



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
import pytest
import salt.states.eselect as eselect
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {eselect: {}}


def test_set_():
    """
    Test to verify that the given module is set to the given target
    """
    name = "myeselect"
    target = "hardened/linux/amd64"

    ret = {"name": name, "result": True, "comment": "", "changes": {}}

    mock = MagicMock(return_value=target)
    with patch.dict(eselect.__salt__, {"eselect.get_current_target": mock}):
        comt = "Target '{}' is already set on '{}' module.".format(target, name)
        ret.update({"comment": comt})
        assert eselect.set_(name, target) == ret



import datetime
import logging
import os
import signal
import subprocess
import textwrap
import time

import pytest
import salt.states.file
import salt.utils.files
import salt.utils.path
import salt.utils.platform
from tests.support.case import ModuleCase
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


@pytest.mark.skip_unless_on_linux
class SystemModuleTest(ModuleCase):
    """
    Validate the date/time functions in the system module
    """

    _hwclock_has_compare_ = None
    _systemd_timesyncd_available_ = None

    @classmethod
    def setUpClass(cls):
        cls.fmt_str = "%Y-%m-%d %H:%M:%S"
        cls._orig_time = None
        cls._machine_info = True

    @classmethod
    def tearDownClass(cls):
        for name in ("fmt_str", "_orig_time", "_machine_info"):
            delattr(cls, name)

    def setUp(self):
        super().setUp()
        if self._systemd_timesyncd_available_ is None:
            SystemModuleTest._systemd_timesyncd_available_ = self.run_function(
                "service.available", ["systemd-timesyncd"]
            )
        if self._systemd_timesyncd_available_:
            self.run_function("service.stop", ["systemd-timesyncd"])

    def tearDown(self):
        if self._orig_time is not None:
            self._restore_time()
        self._orig_time = None
        if self._machine_info is not True:
            self._restore_machine_info()
        self._machine_info = True
        if self._systemd_timesyncd_available_:
            self.run_function("service.start", ["systemd-timesyncd"])

    def _save_time(self):
        self._orig_time = datetime.datetime.utcnow()

    def _set_time(self, new_time, offset=None):
        t = new_time.timetuple()[:6]
        t += (offset,)
        return self.run_function("system.set_system_date_time", t)

    def _restore_time(self):
        result = self._set_time(self._orig_time, "+0000")
        self.assertTrue(result, msg="Unable to restore time properly")

    def _same_times(self, t1, t2, seconds_diff=30):
        """
        Helper function to check if two datetime objects
        are close enough to the same time.
        """
        return abs(t1 - t2) < datetime.timedelta(seconds=seconds_diff)

    def _hwclock_has_compare(self):
        """
        Some builds of hwclock don't include the `--compare` function
        needed to test hw/sw clock synchronization. Returns false on
        systems where it's not present so that we can skip the
        comparison portion of the test.
        """
        if self._hwclock_has_compare_ is None:
            res = self.run_function("cmd.run_all", cmd="hwclock -h")
            SystemModuleTest._hwclock_has_compare_ = (
                res["retcode"] == 0 and res["stdout"].find("--compare") > 0
            )
        return self._hwclock_has_compare_

    def _test_hwclock_sync(self):
        """
        Check that hw and sw clocks are sync'd.
        """
        if not self.run_function("system.has_settable_hwclock"):
            return None
        if not self._hwclock_has_compare():
            return None

        class CompareTimeout(BaseException):
            pass

        def _alrm_handler(sig, frame):
            log.warning("hwclock --compare failed to produce output after 3 seconds")
            raise CompareTimeout

        for _ in range(2):
            try:
                orig_handler = signal.signal(signal.SIGALRM, _alrm_handler)
                signal.alarm(3)
                rpipeFd, wpipeFd = os.pipe()
                log.debug("Comparing hwclock to sys clock")
                with os.fdopen(rpipeFd, "r") as rpipe:
                    with os.fdopen(wpipeFd, "w") as wpipe:
                        with salt.utils.files.fopen(os.devnull, "r") as nulFd:
                            p = subprocess.Popen(
                                args=["hwclock", "--compare"],
                                stdin=nulFd,
                                stdout=wpipeFd,
                                stderr=subprocess.PIPE,
                            )
                            p.communicate()

                            # read header
                            rpipe.readline()

                            # read first time comparison
                            timeCompStr = rpipe.readline()

                            # stop
                            p.terminate()

                            timeComp = timeCompStr.split()
                            hwTime = float(timeComp[0])
                            swTime = float(timeComp[1])
                            diff = abs(hwTime - swTime)

                            self.assertTrue(
                                diff <= 2.0,
                                msg=("hwclock difference too big: " + str(timeCompStr)),
                            )
                            break
            except CompareTimeout:
                p.terminate()
            finally:
                signal.alarm(0)
                signal.signal(signal.SIGALRM, orig_handler)
        else:
            log.error("Failed to check hwclock sync")

    def _save_machine_info(self):
        if os.path.isfile("/etc/machine-info"):
            with salt.utils.files.fopen("/etc/machine-info", "r") as mach_info:
                self._machine_info = mach_info.read()
        else:
            self._machine_info = False

    def _restore_machine_info(self):
        if self._machine_info is not False:
            with salt.utils.files.fopen("/etc/machine-info", "w") as mach_info:
                mach_info.write(self._machine_info)
        else:
            self.run_function("file.remove", ["/etc/machine-info"])

    @pytest.mark.slow_test
    def test_get_system_date_time(self):
        """
        Test we are able to get the correct time
        """
        t1 = datetime.datetime.now()
        res = self.run_function("system.get_system_date_time")
        t2 = datetime.datetime.strptime(res, self.fmt_str)
        msg = "Difference in times is too large. Now: {} Fake: {}".format(t1, t2)
        self.assertTrue(self._same_times(t1, t2, seconds_diff=2), msg=msg)

    @pytest.mark.slow_test
    def test_get_system_date_time_utc(self):
        """
        Test we are able to get the correct time with utc
        """
        t1 = datetime.datetime.utcnow()
        res = self.run_function("system.get_system_date_time", utc_offset="+0000")
        t2 = datetime.datetime.strptime(res, self.fmt_str)
        msg = "Difference in times is too large. Now: {} Fake: {}".format(t1, t2)
        self.assertTrue(self._same_times(t1, t2, seconds_diff=2), msg=msg)

    @pytest.mark.destructive_test
    @pytest.mark.skip_if_not_root
    @pytest.mark.slow_test
    def test_set_system_date_time(self):
        """
        Test changing the system clock. We are only able to set it up to a
        resolution of a second so this test may appear to run in negative time.
        """
        self._save_time()
        cmp_time = datetime.datetime.now() - datetime.timedelta(days=7)
        result = self._set_time(cmp_time)
        time_now = datetime.datetime.now()

        msg = "Difference in times is too large. Now: {} Fake: {}".format(
            time_now, cmp_time
        )
        self.assertTrue(result and self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @pytest.mark.destructive_test
    @pytest.mark.skip_if_not_root
    @pytest.mark.slow_test
    def test_set_system_date_time_utc(self):
        """
        Test changing the system clock. We are only able to set it up to a
        resolution of a second so this test may appear to run in negative time.
        """
        self._save_time()
        cmp_time = datetime.datetime.utcnow() - datetime.timedelta(days=7)
        result = self._set_time(cmp_time, offset="+0000")
        time_now = datetime.datetime.utcnow()

        msg = "Difference in times is too large. Now: {} Fake: {}".format(
            time_now, cmp_time
        )
        self.assertTrue(result)
        self.assertTrue(self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @pytest.mark.destructive_test
    @pytest.mark.skip_if_not_root
    @pytest.mark.slow_test
    def test_set_system_date_time_utcoffset_east(self):
        """
        Test changing the system clock. We are only able to set it up to a
        resolution of a second so this test may appear to run in negative time.
        """
        self._save_time()
        cmp_time = datetime.datetime.utcnow() - datetime.timedelta(days=7)
        # 25200 seconds = 7 hours
        time_to_set = cmp_time - datetime.timedelta(seconds=25200)
        result = self._set_time(time_to_set, offset="-0700")
        time_now = datetime.datetime.utcnow()

        msg = "Difference in times is too large. Now: {} Fake: {}".format(
            time_now, cmp_time
        )
        self.assertTrue(result)
        self.assertTrue(self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @pytest.mark.destructive_test
    @pytest.mark.skip_if_not_root
    @pytest.mark.slow_test
    def test_set_system_date_time_utcoffset_west(self):
        """
        Test changing the system clock. We are only able to set it up to a
        resolution of a second so this test may appear to run in negative time.
        """
        self._save_time()
        cmp_time = datetime.datetime.utcnow() - datetime.timedelta(days=7)
        # 7200 seconds = 2 hours
        time_to_set = cmp_time + datetime.timedelta(seconds=7200)
        result = self._set_time(time_to_set, offset="+0200")
        time_now = datetime.datetime.utcnow()

        msg = "Difference in times is too large. Now: {} Fake: {}".format(
            time_now, cmp_time
        )
        self.assertTrue(result)
        self.assertTrue(self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @pytest.mark.flaky(max_runs=4)
    @pytest.mark.destructive_test
    @pytest.mark.skip_if_not_root
    @pytest.mark.slow_test
    def test_set_system_time(self):
        """
        Test setting the system time without adjusting the date.
        """
        cmp_time = datetime.datetime.now().replace(hour=10, minute=5, second=0)
        self._save_time()

        result = self.run_function("system.set_system_time", ["10:05:00"])

        time_now = datetime.datetime.now()
        msg = "Difference in times is too large. Now: {} Fake: {}".format(
            time_now, cmp_time
        )

        self.assertTrue(result)
        self.assertTrue(self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @pytest.mark.destructive_test
    @pytest.mark.skip_if_not_root
    @pytest.mark.slow_test
    def test_set_system_date(self):
        """
        Test setting the system date without adjusting the time.
        """
        cmp_time = datetime.datetime.now() - datetime.timedelta(days=7)

        self._save_time()
        result = self.run_function(
            "system.set_system_date", [cmp_time.strftime("%Y-%m-%d")]
        )

        time_now = datetime.datetime.now()
        msg = "Difference in times is too large. Now: {} Fake: {}".format(
            time_now, cmp_time
        )

        self.assertTrue(result)
        self.assertTrue(self._same_times(time_now, cmp_time), msg=msg)
        self._test_hwclock_sync()

    @pytest.mark.skip_if_not_root
    @pytest.mark.slow_test
    def test_get_computer_desc(self):
        """
        Test getting the system hostname
        """
        res = self.run_function("system.get_computer_desc")

        hostname_cmd = salt.utils.path.which("hostnamectl")
        if hostname_cmd:
            desc = self.run_function("cmd.run", ["hostnamectl status --pretty"])
            self.assertEqual(res, desc)
        else:
            if not os.path.isfile("/etc/machine-info"):
                self.assertFalse(res)
            else:
                with salt.utils.files.fopen("/etc/machine-info", "r") as mach_info:
                    data = mach_info.read()
                    self.assertIn(res, data.decode("string_escape"))

    @pytest.mark.destructive_test
    @pytest.mark.skip_if_not_root
    @pytest.mark.slow_test
    def test_set_computer_desc(self):
        """
        Test setting the computer description
        """
        self._save_machine_info()
        desc = "test"
        ret = self.run_function("system.set_computer_desc", [desc])
        computer_desc = self.run_function("system.get_computer_desc")

        self.assertTrue(ret)
        self.assertIn(desc, computer_desc)

    @pytest.mark.destructive_test
    @pytest.mark.skip_if_not_root
    @pytest.mark.slow_test
    def test_set_computer_desc_multiline(self):
        """
        Test setting the computer description with a multiline string with tabs
        and double-quotes.
        """
        self._save_machine_info()
        desc = textwrap.dedent(
            '''\
            'First Line
            \tSecond Line: 'single-quoted string'
            \t\tThird Line: "double-quoted string with unicode: питон"'''
        )
        ret = self.run_function("system.set_computer_desc", [desc])
        # self.run_function returns the serialized return, we need to convert
        # back to unicode to compare to desc. in the assertIn below.
        computer_desc = salt.utils.stringutils.to_unicode(
            self.run_function("system.get_computer_desc")
        )

        self.assertTrue(ret)
        self.assertIn(desc, computer_desc)

    @pytest.mark.skip_if_not_root
    @pytest.mark.slow_test
    def test_has_hwclock(self):
        """
        Verify platform has a settable hardware clock, if possible.
        """
        if self.run_function("grains.get", ["os_family"]) == "NILinuxRT":
            self.assertTrue(self.run_function("system._has_settable_hwclock"))
            self.assertTrue(self._hwclock_has_compare())


@pytest.mark.skip_unless_on_windows
@pytest.mark.windows_whitelisted
class WinSystemModuleTest(ModuleCase):
    """
    Validate the date/time functions in the win_system module
    """

    @pytest.mark.slow_test
    def test_get_computer_name(self):
        """
        Test getting the computer name
        """
        ret = self.run_function("system.get_computer_name")

        self.assertTrue(isinstance(ret, str))
        import socket

        name = socket.gethostname()
        self.assertEqual(name, ret)

    @pytest.mark.destructive_test
    @pytest.mark.slow_test
    def test_set_computer_desc(self):
        """
        Test setting the computer description
        """
        current_desc = self.run_function("system.get_computer_desc")
        desc = "test description"
        try:
            set_desc = self.run_function("system.set_computer_desc", [desc])
            self.assertTrue(set_desc)

            get_desc = self.run_function("system.get_computer_desc")
            self.assertEqual(set_desc["Computer Description"], get_desc)
        finally:
            self.run_function("system.set_computer_desc", [current_desc])

    @skipIf(True, "WAR ROOM 7/29/2019, unit test?")
    def test_get_system_time(self):
        """
        Test getting the system time
        """
        time_now = datetime.datetime.now()
        # We have to do some datetime fu to account for the possibility that the
        # system time will be obtained just before the minutes increment
        ret = self.run_function("system.get_system_time", timeout=300)
        # Split out time and AM/PM
        sys_time, meridian = ret.split(" ")
        h, m, s = sys_time.split(":")
        # Get the current time
        # Use the system time to generate a datetime object for the system time
        # with the same date
        time_sys = time_now.replace(hour=int(h), minute=int(m), second=int(s))
        # get_system_time returns a non 24 hour time
        # Lets make it 24 hour time
        if meridian == "PM":
            time_sys = time_sys + datetime.timedelta(hours=12)
        diff = time_sys - time_now
        # Timeouts are set to 300 seconds. We're adding a 30 second buffer
        self.assertTrue(diff.seconds < 330)

    @pytest.mark.slow_test
    def test_get_system_date(self):
        """
        Test getting system date
        """
        ret = self.run_function("system.get_system_date")
        date = datetime.datetime.now().strftime("%m/%d/%Y")
        self.assertEqual(date, ret)


@pytest.mark.skip_unless_on_windows
@pytest.mark.windows_whitelisted
class WinSystemModuleTimeSettingTest(ModuleCase):
    """
    Validate the date/time functions in the win_system module

    .. note::

        In order for these tests to pass, time sync must be disabled for the VM
        in the hyper-visor
    """

    @classmethod
    def setUpClass(cls):
        if subprocess.call("sc config w32time start= disabled > nul", shell=True) != 0:
            log.error("Failed to disable w32time service")
        if subprocess.call("sc stop w32time", shell=True) != 0:
            log.error("Failed to stop w32time service")

    @classmethod
    def tearDownClass(cls):
        if subprocess.call("sc config w32time start= auto > nul", shell=True) != 0:
            log.error("Failed to enable w32time service")
        if subprocess.call("sc start w32time", shell=True) != 0:
            log.error("Failed to start w32time service")
        if subprocess.call("w32tm /resync", shell=True) != 0:
            log.error("Re-syncing time failed")

    @skipIf(True, "WAR ROOM 7/18/2019, unit test?")
    @pytest.mark.destructive_test
    @pytest.mark.slow_test
    def test_set_system_time(self):
        """
        Test setting the system time
        """
        # If the test fails, the hypervisor may be maintaining time sync
        test_time = "10:55"
        self.run_function("system.set_system_time", [test_time + " AM"])
        time.sleep(0.25)
        new_time = datetime.datetime.now().strftime("%H:%M")
        self.assertEqual(new_time, test_time)

    @skipIf(True, "WAR ROOM 7/18/2019, unit test?")
    @pytest.mark.destructive_test
    @pytest.mark.slow_test
    def test_set_system_date(self):
        """
        Test setting system date
        """
        # If the test fails, the hypervisor may be maintaining time sync
        self.run_function("system.set_system_date", ["03/25/2018"])
        new_date = datetime.datetime.now().strftime("%Y/%m/%d")
        self.assertEqual(new_date, "2018/03/25")



"""
Return data to an etcd server or cluster

:depends: - python-etcd

In order to return to an etcd server, a profile should be created in the master
configuration file:

.. code-block:: yaml

    my_etcd_config:
      etcd.host: 127.0.0.1
      etcd.port: 2379

It is technically possible to configure etcd without using a profile, but this
is not considered to be a best practice, especially when multiple etcd servers
or clusters are available.

.. code-block:: yaml

    etcd.host: 127.0.0.1
    etcd.port: 2379

Additionally, two more options must be specified in the top-level configuration
in order to use the etcd returner:

.. code-block:: yaml

    etcd.returner: my_etcd_config
    etcd.returner_root: /salt/return

The ``etcd.returner`` option specifies which configuration profile to use. The
``etcd.returner_root`` option specifies the path inside etcd to use as the root
of the returner system.

Once the etcd options are configured, the returner may be used:

CLI Example:

    salt '*' test.ping --return etcd

A username and password can be set:

.. code-block:: yaml

    etcd.username: larry  # Optional; requires etcd.password to be set
    etcd.password: 123pass  # Optional; requires etcd.username to be set

You can also set a TTL (time to live) value for the returner:

.. code-block:: yaml

    etcd.ttl: 5

Authentication with username and password, and ttl, currently requires the
``master`` branch of ``python-etcd``.

You may also specify different roles for read and write operations. First,
create the profiles as specified above. Then add:

.. code-block:: yaml

    etcd.returner_read_profile: my_etcd_read
    etcd.returner_write_profile: my_etcd_write
"""

import logging

import salt.utils.jid
import salt.utils.json

try:
    import salt.utils.etcd_util

    HAS_LIBS = True
except ImportError:
    HAS_LIBS = False

log = logging.getLogger(__name__)

# Define the module's virtual name
__virtualname__ = "etcd"


def __virtual__():
    """
    Only return if python-etcd is installed
    """
    if HAS_LIBS:
        return __virtualname__

    return False, "Could not import etcd returner; python-etcd is not installed."


def _get_conn(opts, profile=None):
    """
    Establish a connection to etcd
    """
    if profile is None:
        profile = opts.get("etcd.returner")
    path = opts.get("etcd.returner_root", "/salt/return")
    return salt.utils.etcd_util.get_conn(opts, profile), path


def returner(ret):
    """
    Return data to an etcd server or cluster
    """
    write_profile = __opts__.get("etcd.returner_write_profile")
    if write_profile:
        ttl = __opts__.get(write_profile, {}).get("etcd.ttl")
    else:
        ttl = __opts__.get("etcd.ttl")

    client, path = _get_conn(__opts__, write_profile)
    # Make a note of this minion for the external job cache
    client.set(
        "/".join((path, "minions", ret["id"])),
        ret["jid"],
        ttl=ttl,
    )

    for field in ret:
        # Not using os.path.join because we're not dealing with file paths
        dest = "/".join((path, "jobs", ret["jid"], ret["id"], field))
        client.set(dest, salt.utils.json.dumps(ret[field]), ttl=ttl)


def save_load(jid, load, minions=None):
    """
    Save the load to the specified jid
    """
    log.debug("sdstack_etcd returner <save_load> called jid: %s", jid)
    write_profile = __opts__.get("etcd.returner_write_profile")
    client, path = _get_conn(__opts__, write_profile)
    if write_profile:
        ttl = __opts__.get(write_profile, {}).get("etcd.ttl")
    else:
        ttl = __opts__.get("etcd.ttl")
    client.set(
        "/".join((path, "jobs", jid, ".load.p")),
        salt.utils.json.dumps(load),
        ttl=ttl,
    )


def save_minions(jid, minions, syndic_id=None):  # pylint: disable=unused-argument
    """
    Included for API consistency
    """


def clean_old_jobs():
    """
    Included for API consistency
    """


def get_load(jid):
    """
    Return the load data that marks a specified jid
    """
    log.debug("sdstack_etcd returner <get_load> called jid: %s", jid)
    read_profile = __opts__.get("etcd.returner_read_profile")
    client, path = _get_conn(__opts__, read_profile)
    return salt.utils.json.loads(
        client.get("/".join((path, "jobs", jid, ".load.p"))).value
    )


def get_jid(jid):
    """
    Return the information returned when the specified job id was executed
    """
    log.debug("sdstack_etcd returner <get_jid> called jid: %s", jid)
    ret = {}
    client, path = _get_conn(__opts__)
    items = client.get("/".join((path, "jobs", jid)))
    for item in items.children:
        if str(item.key).endswith(".load.p"):
            continue
        comps = str(item.key).split("/")
        data = client.get("/".join((path, "jobs", jid, comps[-1], "return"))).value
        ret[comps[-1]] = {"return": salt.utils.json.loads(data)}
    return ret


def get_fun(fun):
    """
    Return a dict of the last function called for all minions
    """
    log.debug("sdstack_etcd returner <get_fun> called fun: %s", fun)
    ret = {}
    client, path = _get_conn(__opts__)
    items = client.get("/".join((path, "minions")))
    for item in items.children:
        comps = str(item.key).split("/")
        efun = salt.utils.json.loads(
            client.get(
                "/".join((path, "jobs", str(item.value), comps[-1], "fun"))
            ).value
        )
        if efun == fun:
            ret[comps[-1]] = str(efun)
    return ret


def get_jids():
    """
    Return a list of all job ids
    """
    log.debug("sdstack_etcd returner <get_jids> called")
    ret = []
    client, path = _get_conn(__opts__)
    items = client.get("/".join((path, "jobs")))
    for item in items.children:
        if item.dir is True:
            jid = str(item.key).split("/")[-1]
            ret.append(jid)
    return ret


def get_minions():
    """
    Return a list of minions
    """
    log.debug("sdstack_etcd returner <get_minions> called")
    ret = []
    client, path = _get_conn(__opts__)
    items = client.get("/".join((path, "minions")))
    for item in items.children:
        comps = str(item.key).split("/")
        ret.append(comps[-1])
    return ret


def prep_jid(nocache=False, passed_jid=None):  # pylint: disable=unused-argument
    """
    Do any work necessary to prepare a JID, including sending a custom id
    """
    return passed_jid if passed_jid is not None else salt.utils.jid.gen_jid(__opts__)
